/**
 *
 * Description : A class that will post the events using CP Event & Message Service provider. 
 * <p>
 * Filename    : SSOServiceManager.js
 * <p>
 * Part of     : services 
 * <p>
 * ============================================================================
 * <p>
 * Copyright (C) 2008 Nokia.  All rights reserved.
 * <p>
 * This material, including documentation and any related computer
 * programs, is protected by copyright controlled by Nokia.  All
 * rights are reserved.  Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the
 * prior written consent of Nokia.  This material also contains
 * confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 * <p>
 * ============================================================================
 * @class SSOServiceManager
 * @name SSOServiceManager 
 */
 
ovi.provide('ssoui.services.SSOServiceManager');
ovi.require('ovi.service.sfwapi', false );
ovi.require('ovi.service.sfwiter', false );
ovi.require('ovi.service.exceptions', false );
ovi.require('ovi.service.sso.SSO', false);
ovi.require('ssoui.services.SSOEventManager');
ovi.require('ssoui.utils.SSOConstant',false)
ovi.require('ssoui.utils.SSOUtil',false);

ovi.declare('ssoui.services.SSOServiceManager', {
		
	/**
	 * Creates the objects needed for this controller
	 * Calls the base class init function which initiliazes E & M service provider
	 * @function initialize
	 */
    initialize: function(aCmdConfig, aCallback, aContext) {
		this._ssoEventManager = null;
		this._sysInfo = null;
		this._noaAuthenication = null;
		this._noaRegistration = null;
		this._noaPasswordManager = null;
		this._noaProfileManager = null;
		this._appManagement = null;
		this._callback = aCallback;
		this._context = aContext;
		this._loadServiceProviders(aCmdConfig);
    },
	
    
    /**
     * Loads all the service providers required by the SSO JS UI.
     * @function _loadServiceProviders
     */
	_loadServiceProviders: function(aCmdConfig, aCallback) {
    	try {
    		
	        this._consumerKey = aCmdConfig.ssoConsumerKey;
	        this._consumerSecret = aCmdConfig.ssoConsumerSecret;
	        this._usecase = aCmdConfig.ssoUIContext;
	        this._baseurl = aCmdConfig.ssobaseurl;
	        
	        if (typeof aCmdConfig.ssobaseurl == 'undefined') {        
	            this._baseurl = 2;
	        } else {
	            this._baseurl = aCmdConfig.ssobaseurl; 
	        }
	        
	        var serviceId = aCmdConfig.ssoServiceId;
	        
	        // activate the SSOEventManager class.
	        this._ssoEventManager =  new ssoui.services.SSOEventManager();
	        
	        //Initialize the enabler APIs	        
	        ovi.debug("Started loading enabler objects");	        
	        //Load the authentication plugin. Its an async call    
	        this._noaAuthenication = new ovi.service.sso.NoaAuthentication(this._initNoaAuthObjs, this);
	        //Load the registration plugin. Its an async call
	        this._noaRegistration = new ovi.service.sso.NoaRegistration(this._initNoaRegObjs, this);
	        //Load the profile manager plugin. Its an async call
			this._noaProfileManager = new ovi.service.sso.NoaProfileManager(this._initNoaProObjs, this);
	        //A counter to keep track if all the plugins are loaded
	      	this._enablerobjCounter = 3;	      	
		} catch(ex) {  
            ovi.require( 'ssoui.ui.SSOUIHelper' );
            SSOUIHelper.showSystemError(this.applicationController,sso_msg_0026);       
		}
    },
    
    /**
     * Function which will be triggered on loading of Authentication object
     * @function _initNoaAuthObjs
     */
    _initNoaAuthObjs: function(aContext) {    	
    	ovi.debug("Inside _initNoaAuthObjs");    	
    	//Get the baseurl from the config element
        var baseurl = aContext._baseurl;
        aContext._noaAuthenication.initialiseAsync(aContext._initNoaEnablerObjs, aContext._consumerKey,aContext._consumerSecret,new ConnectionDetail(baseurl), aContext);
    },
    
    /**
     * Function which will be triggered on loading of Registeration object
     * @function _initNoaRegObjs
     */
    _initNoaRegObjs: function(aContext) {
    	ovi.debug("Inside _initNoaRegObjs");    	
    	//Get the baseurl from the config element
        var baseurl = aContext._baseurl;
        aContext._noaRegistration.initialiseAsync(aContext._initNoaEnablerObjs, aContext._consumerKey,aContext._consumerSecret,new ConnectionDetail(baseurl), aContext);
    },
    
    /**
     * Function which will be triggered on loading of profile manager object
     * @function _initNoaProObjs
     */
    _initNoaProObjs: function(aContext) {
    	ovi.debug("Inside _initNoaProObjs");    	
    	//Get the baseurl from the config element
        var baseurl = aContext._baseurl;
        aContext._noaProfileManager.initialiseAsync(aContext._initNoaEnablerObjs, aContext._consumerKey,aContext._consumerSecret,new ConnectionDetail(baseurl), aContext);
    },
    
    /**
     * Callback API when the enabler plugins are loaded
     * Initializes the plugin objects when loaded
     * @function _initNoaEnablerObjs
     * @param {object} aContext - this pointer
     */
    _initNoaEnablerObjs: function(aContext) {
        //Decrement the counter when its laoded
        aContext._enablerobjCounter--;
        
        //Get the baseurl from the config element
        var baseurl = aContext._baseurl;
        
        //If all the plugins are loaded,then initalize them with the
        //consumer key and the secret        
        if(aContext._enablerobjCounter == 0) {            
            ovi.debug("After loading enabler objects ");
        
            //Launch the wizard as all the objects are loaded
            aContext._callback(aContext._usecase, aContext._context);
            
            //add a slight delay to make the UI rendering smooth.
            setTimeout(function() { aContext._loadRemainingPlugins();  }, 10);
        }                    
    },
    
    /**
     * 
     * @function _loadRemainingPlugins
     */
    _loadRemainingPlugins: function() {
    try {
    	ovi.debug("Inside _loadRemainingPlugins function");
    	
    	//@todo load the remaining plugins when required to save the memory
    	//while loading the plugins give a slight delay to make the UI rendering smooth.
    	var _self = this;
        setTimeout(function() { _self._loadAppManagementPlugin(); }, 5);
        setTimeout(function() { _self._loadSysInfoPlugin(); }, 5);
        setTimeout(function() { _self._loadProfileAndPasswordPlugins(); }, 5);
        
        } catch(ex) {
        ovi.debug(ex);
        // error happened while loading relaining plugins.
        ovi.require( 'ssoui.ui.SSOUIHelper' );
        SSOUIHelper.showSystemError(this.applicationController,sso_msg_0026);       
        }
    },
    
    /**
     * Function to load sys info service provider plugin.
     * @function _loadSysInfoPlugin
     */
    _loadSysInfoPlugin: function() {
    	ovi.debug("Inside _loadSysInfoPlugin function");        
    	ovi.require('ssoui.services.SSOSysInfo');
    	//Init the sysinfo service
    	this._sysInfo =  new ssoui.services.SSOSysInfo();
    },
    
    /**
     * Function to load the application management service provider plugin.
     * @function _loadAppManagementPlugin
     */
    _loadAppManagementPlugin: function() {
    	ovi.debug("Inside _loadAppManagementPlugin function");
    	ovi.require('ovi.service.ApplicationManagement', false);
    	//Load the Application management, this is used for launching context sensitive help
    	this._appManagement = new ovi.service.ApplicationManagement(this, this._appMgtCallback);
    },
    
    /**
     * Function to load profile manager & password manager plugins.
     * @function _loadProfileAndPasswordPlugins
     */
    _loadProfileAndPasswordPlugins: function() {
    	ovi.debug("Inside _loadProfileAndPasswordPlugins function");
    	//A counter to keep track if all the plugins are loaded
      	this._enablerobjCounter = 1;
    	//Load the password manager plugin. Its an async call
        this._noaPasswordManager = new ovi.service.sso.NoaPasswordManager(this._initNoaPassMgrObjs, this);        		
    },
    
    /**
     * Function which will be triggered on loading of password manager object
     * @function _initNoaPassMgrObjs
     */
    _initNoaPassMgrObjs: function(aContext) {
    	ovi.debug("Inside _initNoaPassMgrObjs");    	
    	//Get the baseurl from the config element
        var baseurl = aContext._baseurl;
        aContext._noaPasswordManager.initialiseAsync(aContext._initSecondaryNoaEnablerObjs, aContext._consumerKey, aContext._consumerSecret, new ConnectionDetail(baseurl), aContext);
    },
    
    /**
     * Callback API when the enabler plugins are loaded
     * Initializes the plugin objects when loaded
     * @function _initSecondaryNoaEnablerObjs
     * @param {object} aContext - this pointer
     */    
    _initSecondaryNoaEnablerObjs : function(aContext) {
    	ovi.debug("Inside _initSecondaryNoaEnablerObjs function");
    	//@todo anything related to plugin loading
    },
    
    /**
     * This function is fired on completion of initialization of
     * application management class
     * @funciton _appMgtCallback      
     */     
    _appMgtCallback: function(aRetCode) {      
    	if (aRetCode.code!=0) {
    		ovi.debug("AppMgt loading failed : " + aRetCode.message + " Object Error=" + aRetCode.code);
    	}
    },
    
    /**
     * @function getSSOEventManager
     */
    getSSOEventManager: function() {
    	return this._ssoEventManager;
    },
    
    /**
     * 
     * @function getSysInfo
     */
    getSysInfo: function() {
    	return this._sysInfo;	
    },
    
    /**
     * @function getNoaAuthentication
     */
    getNoaAuthentication: function() {
    	return this._noaAuthenication;
    },
    
    /**
     * @function getNoaRegisteration
     */
    getNoaRegisteration: function() {
    	return this._noaRegistration;
    },
    
    /**
     * 
     * @function getNoaPasswordManager
     */
    getNoaPasswordManager: function() {
    	return this._noaPasswordManager;
    },
    
    /**
     * @function getNoaProfileManager
     */
    getNoaProfileManager: function() {
    	return this._noaProfileManager;
    },
    
    /**
     * @function getApplicationManagement
     */
    getApplicationManagement: function() {
    	return this._appManagement;
    }
    
});