/*
 * Copyright (C) 2009, Nokia gate5 GmbH Berlin
 *
 * These coded instructions, statements, and computer programs contain
 * unpublished proprietary information of Nokia gate5 GmbH, and are
 * copy protected by law. They may not be disclosed to third parties
 * or copied or duplicated in any form, in whole or in part, without
 * the specific, prior written permission of Nokia gate5 GmbH.
 */

ovi.provide("ssoui.ui.SSODialog");

ovi.require("ovi.dom");
ovi.require("ovi.ui.Button");
ovi.require("ovi.ui.Label");
ovi.require("ovi.ui.Snippet");

/**
 * UI Dialog. The dialog is fullscreen and modal.
 * It has a message and a predefined set of two buttons (confirm, cancel).
 *
 * @example
 * ovi.require("ssoui.ui.SSODialog");
 *
 * var dialog = new ovi.ui.Dialog({ message: "An example of ovi.ui.Dialog." }, document.body);
 * dialog.addEventHandler(ovi.ui.Dialog.CLOSE_EVENT, function(e){
 *     ovi.info(e.data);
 * });
 *
 * @publish
 * @mergeDoc ovi.ui.Snippet#initialize
 * @param {String} properties.message The dialog's informational text.
 * @param {String} properties.confirm The dialog's confirm button label.
 * @param {String} properties.cancel The dialog's cancel button label.
 */
ssoui.ui.SSODialog = new ovi.Class({
    Extends: ovi.ui.Snippet,
    Statics: {
        /**
         * The constant for the 'close' event. Can be used to register a listener for this event.
         *
         * @see ovi.ui.Dialog#fireEvent:close
         * @type String
         */
        CLOSE_EVENT: "close"
    },

    /**
     * The default template for the dialog.
     *
     * @type String
     * @see ovi.ui.Snippet
     */
    template: '<div class="ovi_Dialog ovi_modal">' +
                  '<div data-bind-text="message"></div>' +                  
                  '<div data-bind-snippet="ovi.ui.Button:cancelButton" data-bind-prop="text:cancel"></div>' +
              '</div>',
    

    /**
     * Default label for the cancel button.
     *
     * @type String
     */
    cancel: "Cancel",

    /**
     * Provides the same functionality as {@link ovi.ui.Snippet#build} and attaches click handler
     * to the confirm and cancel button.
     *
     * @mergeDoc ovi.ui.Snippet#build
     * @see ovi.ui.Snippet#build
     */
    build: function () {
        this._super.apply(this, arguments);

        var that = this;
        this.cancelButton.addEventHandler(ovi.ui.Button.CLICK_EVENT, function (e) {
            that._close(false);
        });
    },

    /**
     * Fires event {@link ovi.ui.Dialog.CLOSE_EVENT} if confirm or cancel button is clicked passing along
     * a true or false respectively to indicate the selection.
     *
     * @event
     * @name ovi.ui.Dialog#fireEvent:close
     * @param {Object} e the sent event object
     * @param {Boolean} e.data <code>true</code> for confirmation, <code>false</code> for cancellation.
     * @see ovi.EventSource
     */

    /**
     * Event listener for the DOM click event of the buttons. This handler will fire the custom 'close' event.
     *
     * @param {Event} e the received DOM event
     * @see ovi.ui.Dialog#fireEvent:close
     * @see ovi.ui.Dialog.CLOSE_EVENT
     */
    _close: function (confirm) {
        var that = this;
        setTimeout(function () {
            that.fireEvent({
                type: ssoui.ui.SSODialog.CLOSE_EVENT,
                data: confirm
            });
            that.destroy();
        }, 300);
    }
});
