/*
 * Copyright (C) 2009, Nokia gate5 GmbH Berlin
 *
 * These coded instructions, statements, and computer programs contain
 * unpublished proprietary information of Nokia gate5 GmbH, and are
 * copy protected by law. They may not be disclosed to third parties
 * or copied or duplicated in any form, in whole or in part, without
 * the specific, prior written permission of Nokia gate5 GmbH.
 */

ovi.provide('ssoui.ui.SSOMobileInput');

ovi.require('ovi.dom');
ovi.require('ovi.ui.Control');
ovi.require('ovi.ui.TextInput');
ovi.require('ovi.ui.Label');

/**
 * CheckButton UI Control. A check button is a clickable entity with a text label. Its width grows with the length of
 * the label. The check button raises an action, if it is clicked. The check button can have 2 states - checked and
 * unchecked.
 *
 * @publish
 * @class
 * @name ssoui.ui.SSOMobileInput
 * @extends ovi.ui.Control
 *
 * @param {Object} [aProperties={}] The properties object that will be used to configure input component on
 *         initialization. Can contain following fields:
 * @param {string} [aProperties.value=""] The initial value of the component.
 * @param {string} [aProperties.label=undefined] The label displayed next to check box input.
 * @param {string} [aProperties.disabled=false] The property indicating whether control is enabled or not.
 * @param {boolean} [aProperties.checked=false] The initial state of check box input field.
 * @param {Node} [aPlacement] The DOM node element where the component will be placed.
 * @param {string} [aPosition="last"] Relative position to the reference node {@link ovi.ui.Snippet#place}.
 */

/**
 * Fires "selected" event if CheckButton is clicked while not disabled.
 *
 * @event
 * @name ssoui.ui.CheckButton#fireEvent:selected
 * @see ovi.EventSource
 */
ssoui.ui.SSOMobileInput = new ovi.Class({
	
    Extends: ovi.ui.Control,

    /**
     * Default template for the check button, will be used to create DOM nodes
     */
    template: '<div class="sso_mobileinput">'+
    				'<div data-bind-snippet="ovi.ui.Label:sso_HeaderLabel"></div> ' +
    				'<div data-bind-snippet="ovi.ui.Label:sso_ErrorLabel"></div> ' +
    				'<div data-bind-snippet="ovi.ui.TextInput:sso_CountryCode"></div>' +
    				'<div data-bind-snippet="ovi.ui.TextInput:sso_MobileNumber"></div>' +
    				'<div data-bind-snippet="ovi.ui.Label:sso_InfoLabel"></div> ' +
    		  '</div>',   
    
    /**
     * Provides custom accessors.
     *
     * @type Object
     * @see ovi.ui.Snippet
     */
    accessors: {
		
		//labels doesn't need getters		
		headerLabel: {
			set:"_setHeaderLabel"
		},
		
		errorLabel: {
			set :"_setErrorLabel"
		},
		
		infoLabel: {
			set :"_setInfoLabel"
		},
		
		countryCode: {
			get: "_getCountryCode", 
			set :"_setCountryCode"
		},
		
		mobileNumber: {
			get: "_getMobileNumber", 
			set :"_setMobileNumber"
		},
		
		value: {
			get: "_getFullValue"
		},
		
		disabled: {
			set :"_setDisable"
		}
	},
	
	Statics: {
		
        /**
         * The constant for the 'keyup' event. Can be used to register a listener for this event.
         *
         * @see ssoui.ui.SSOMobileInput#fireEvent:keyup
         * @type String
         */
		KEYUP_EVENT: "keyup"
    },
    
    /**
     * Provides the same functionality as {@link ovi.ui.Snippet#build} and attaches blur handler
     * on focus change over the text box
     *
     * @mergeDoc ovi.ui.Snippet#build
     * @see ovi.ui.Snippet#build
     */
    build: function() {
		this._super.apply(this, arguments);
        var self = this;
        
        //Event handler for KEYUP_EVENT
        this.sso_CountryCode.addEventHandler("keyup",
            function() {
                ovi.debug("keyup event received.Trigger event");
                self.fireEvent({
                    type: ssoui.ui.SSOMobileInput.KEYUP_EVENT
                });
                return;
            }
        );
        
        this.sso_MobileNumber.addEventHandler("keyup",
            function() {
                ovi.debug("keyup event received.Trigger event");
                self.fireEvent({
                    type: ssoui.ui.SSOMobileInput.KEYUP_EVENT
                });
                return;
            }
        );
	},
	
    
	/**
	 * Function which will be triggered for setting "headerLabel" property.
	 * @param {String} aValue - The label text.
	 * @function  _setHeaderLabel
	 */
	_setHeaderLabel: function(aValue) {
		this.sso_HeaderLabel.setProperty("text", aValue);
	},
	
	/**
	 * Function which will be triggered for setting "errorLabel" property.
	 * @param {String} aValue - The label text.
	 * @function _setErrorLabel
	 */
	_setErrorLabel: function(aValue) {
		this.sso_HeaderLabel.setProperty("text", aValue);
	},
	
	/**
	 * Function which will be triggered for setting "infoLabel" property.
	 * @param {String} aValue - The label text.
	 * @function _setInfoLabel
	 */
	_setInfoLabel: function(aValue) {
		this.sso_InfoLabel.setProperty("text", aValue);	
	},
	
	/**
	 * Function which will be triggered for getting "countryCode" property.
	 * @function _getCountryCode
	 */
	_getCountryCode: function() {
		return this.sso_CountryCode.getProperty("value");
	},
	
	/**
	 * Function which will be triggered for setting "countryCode" property.
	 * @param {String} aValue - country code.
	 * @function _setCountryCode
	 */
	_setCountryCode: function(aValue) {
		this.sso_CountryCode.setProperty("value", aValue);
	},
	
	
	/**
	 * Function which will be triggered for getting "mobileNumber" property.
	 * @function _getMobileNumber
	 */
	_getMobileNumber: function() {
		return this.sso_MobileNumber.getProperty("value");
	},
	
	/**
	 * Function which will be triggered for setting "mobileNumber" property.
	 * @param {String} aValue - Mobile number.
	 * @function _setMobileNumber
	 */
	_setMobileNumber: function(aValue) {
		this.sso_MobileNumber.setProperty("value", aValue);
	},
	
	/**
	 * Function which will be triggered for setting "value" property.
	 * @function _getFullValue
	 */
	_getFullValue: function() {
		return this.sso_CountryCode.getProperty("value")+""+this.sso_MobileNumber.getProperty("value");
	},
	
	/**
	 * Function which will be triggered for setting "disabled" property.
	 * @param {Boolean} aValue - If the input should be disabled or not. 
	 * @function _setDisable
	 */
	_setDisable: function(aValue) {
		this.sso_CountryCode.setProperty("disabled", aValue);
		this.sso_MobileNumber.setProperty("disabled", aValue);
	},
	
    				
    /**
     * Initialization function of the mobile input control.
	 * @param {String} aProperties - mobile field properties.
	 * @param {String} aPlacement - container holding the element.
     * @function initialize
     */				
    initialize : function(aProperties, aPlacement) {		
		this._super.apply(this, arguments);

		this.sso_HeaderLabel.setProperty("extraClass","sso_field_Label");	
		this.sso_CountryCode.setProperty("extraClass","sso_countryCode");
		this.sso_MobileNumber.setProperty("extraClass","sso_mobileNumber");
		this.sso_InfoLabel.setProperty("extraClass","sso_field_ins_Label");
	}    
});
