/**
 *
 * Description : A static class (Javascript object) that maintains a set of
 * properties within the wrtkit framework.  It is used to hold variables that 
 * should be available for all the classes in the framework.
 * <p>
 * Filename    : SSOUtil.js
 * <p>
 * Part of     : UI Framework
 * <p>
 * ============================================================================
 * <p>
 * Copyright (C) 2008 Nokia.  All rights reserved.
 * <p>
 * This material, including documentation and any related computer
 * programs, is protected by copyright controlled by Nokia.  All
 * rights are reserved.  Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the
 * prior written consent of Nokia.  This material also contains
 * confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 * <p>
 * ============================================================================
 * @version %version:  12.1.2 %
 * 
 * @memberOf ovi.serviceui.ssoui.utils
 */

if(typeof ssoui =="undefined") {
	ssoui = {};
}	

if(typeof ssoui.utils =="undefined") {
	ssoui.utils = {};
}

ssoui.utils.SSOUtil = {/** @lends SSOUtil */
		
		/**
		 * Function to check if the passed in object is a function.
		 * @function isFunction
		 * @param {Object} 
		 * @return {Boolean} 
		 */
		isFunction: function(_object) {
			var result = false;			
			if(typeof(_object)=="function")
				result = true;
			
			return result;
		},
		
		/**
		 *  This checks whether the passed string is empty or not
		 * @function  isEmpty
		 * @param {String} 
		 * @return {Boolean}
		 */
		isEmpty: function(_string) {
			//@TODO: CR: Ensure passed data is string.If not return false
			_string = this.trim(_string);
			if(_string.length > 0)	{
				return false;
			}
			
			return true;
		},

        /**
         * Checks whether the answer to the security question is valid or not. 
         * @function validateSecQuestionAnswer
         * @param {String} userName The answer to be validated.
         * @param {String} error message to be displayed.	
         * @return {Boolean} true on success, false on failure
         */
        validateSecQuestionAnswer: function(answer, errorMessages) {
            
            //Trim the answer before validation
            var answerString = this.trim(answer);
            
            if(answerString.length < 1)	{                
                errorMessages.Message = sso_msg_0015;
                return false;
            }  

            // regular expression for checking invalid characters
            // invalid charaters will be .., *, `, |, <, >, /
            var regex = /^.*\.\..*$|[*`|<>/]/;			
            if(answerString.match(regex)){            
                // setting the error messages
                errorMessages.Message = sso_msg_0019;
                return false;
            } 
            // answer is valid           
            return true;
        },
		
		//@TODO: CR: MEDIUM: Below is private method
		/**
		 *  This function trims the white spaces
		 * @function  trim
		 * @param {String}
		 * @return {Boolean}
		 */
		trim: function(_string) {
			return _string.replace(/^\s+|\s+$/g,"");
		},

		/**
		 * Checks whether the sent username is valid or not 
		 * @function validateUserName
		 * @param {String} userName The userName to be validated.
		 * @param {String} error message to be displayed.	
		 * @return {Boolean} true on success, false on failure
		 */
		validateUserName: function(userName, errorMessages) {
                      
            /* Regular expressions to prevent not-allowed cases 
                1. starting with'PA-' 
                2. starting with '.'
                3. ending with '.'
                4. containing '..'
             */  
            var regExNotAllowed = /^PA-.*$|^\..*$|^.*\.$|^.*\.\..*$/;

            // Regular expressing for identifying cases with only numbers
            var regExDigits = /[^0-9]/;
            // valid user name
            var validRegEx = /^[a-zA-Z0-9_.-]*$/;

             // Verify length of user name.
            if( userName.length <4 || userName.length >32) {                
                // User name length is not appropriate. Display error
                errorMessages.Message = sso_msg_username_short;
                return false;
            }

            if(userName.match(validRegEx) ) { 
                // given name matched with the valid regular expression.
                // But there might be a chance that it is still invalid
                // Some more checks need to be done.

                if(userName.match(regExNotAllowed) ) {                    
                    // User name is not allowed.
                    // TODO: Replace with an appropriate error message in future.
                    // (No appropriate error message is available now)
                    errorMessages.Message = sso_msg_username_invalid;
                    return false;
                }

                if (!userName.match(regExDigits)) {                     
                    // User name contain only digits. It is not allowed.
                    // TODO: Error message must be "Must contain at least one non-numeric character".
                    // (Error message is not available. Hence using sso_msg_0002)
                    errorMessages.Message = sso_msg_username_invalid;
                    return false;
                }
                // User name has passed all validity checks.
                return true;
            }            

            // User name is invalid.
            // Given name is containing invalid characters.
            errorMessages.Message = sso_msg_username_invalid;
            return false;
		},
		
		/**
		* Checks whether the sent mobile number is valid or not 
		 * @function validatePassword
		 * @param {String} password The password to be validated
		 * @param {String} error message to be displayed.	
		 * @return {Boolean} true on success, false on failure
		 */
		validatePassword: function(password,errorMessages) {
			var regexp = /^.*[\s]+.*$|[*`|<>/]/;           
			if(password.length < 6 || password.length > 18) {
				errorMessages.Message = sso_msg_pwd_short;
				return false;
			} else if(password.match(regexp)) {
				errorMessages.Message = sso_msg_pwd_invalid;
				return false;
			} else {
				return true;
			}
		},
		
		/**
		 * Checks whether the sent mobile number is valid or not 
		 * @function validateMobileNumber
		 * @param {string} mobileNumber The mobilenumber to be validated	
		 * @return {boolean} true on success, false on failure		 
		 */
		//@TODO: CR: CRITICAL: What happens if '+' is in between. Below 
		//    validation will pass. 
		//@TODO: CR: CRITICAL: Ideally, the control should have
		//        input of type number. To avoid input of non-numbers 
		//    the number?
		validateMobileNumber: function(mobileNumber) {
			mobileNumber = new String(mobileNumber);
			if((mobileNumber.length > 9) && (mobileNumber.length < 19) && (mobileNumber.indexOf("+")) != -1){  					
				if(isNaN(mobileNumber)) {					
					return false;
				} else {
					return true;
				}
            }            
			return false;            
		},
		
		/**
		 * Checks whether the sent emaild id is valid or not
		 * @function validateEmailId
		 * @param {string} emailid The mailid to be validated	
		 * @return {boolean} true on success, false on failure     	 
		 */
		validateEmailId: function(emailId) {
			emailId = new String(emailId);
			if(emailId.length <= 50) {
				var emailReg = /^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,4})+$/;
				var result = emailReg.test(emailId); 			
				return result;
			}
			return false;
		},
        
        /**
		 * Checks whether the sent date id is valid or not
		 * @function validateDateOfBirth
		 * @param {string} aDay The day to be validated
         * @param {string} aMonth The month to be validated
         * @param {string} aYear The year to be validated
		 * @return {boolean} true on success, false on failure     	 
		 */
        validateDateOfBirth: function(aDay,aMonth,aYear) {            
            //Check if the date is a digit
            if(isNaN(aDay) || (isNaN(aMonth) || (isNaN(aYear)))) {
                return false;
            }
            //Get the current date from the phone. Reads the date from the phone settings.
             var currentTime = new Date();
             //Read the month from the current time
             var currentMonth = currentTime.getMonth() + 1;            
             //Read the day from the current time
             var currentDay = currentTime.getDate();
            //Read the year from the current time            
            var currentYear = currentTime.getFullYear();
            //Prepare the days array for a month
            var no_of_days = this.daysArray();
            
            //Validate the month
            if (aMonth.length < 1 || aMonth < 1 || aMonth > 12 ){                    
                    return false;
            }
            //Validate the day
            if (aDay.length < 1 || aDay<1 || aDay>31 || (aMonth == 2 && aDay > ssoui.utils.SSOUtil.daysInFebruary(aYear)) || aDay > no_of_days[aMonth]){                
                return false;
            }
            //Validate the year
            if (aYear.length != 4 || aYear==0 || aYear > currentYear){                  
                  return false;
            }  
            // Fix for KSSS-82YC74 
            // Validation has been done for day, month and year entries.
            // Chance of a future year has already been ruled out.
            // Validate if the date and month is valid as per the current year.
            if( aYear == currentYear )
            {
                if( (aMonth > currentMonth) || ( (aMonth == currentMonth) && (aDay > currentDay) ) )
                {
                    //future month or day in the current year
                    return false;
                }                
            }
            return true;
        },
        
		//@TODO: CR: MEDIUM. Make below function private. Use Prefix "_"
        daysInFebruary: function (aYear){
            // February has 29 days in any year evenly divisible by four,
            // EXCEPT for centurial years which are not also divisible by 400.
            return (((aYear % 4 == 0) && ( (!(aYear % 100 == 0)) || (aYear % 400 == 0))) ? 29 : 28 );
        },
        
        //@TODO: CR: MEDIUM. Make below function private. Use Prefix "_"
        daysArray: function() {
             var days = new Array;
            //Prepare the days array for all the months
            for (var i = 1; i <= 12; i++) {
                //Prepare the days array to 31 days
                 days[i] = 31
                //If the month is april,june,Sep,Nov then the number of days is 30
                if (i==4 || i==6 || i==9 || i==11) {
                    days[i] = 30;
                }
                //If the month is feb then the number of days is 29.
                //Leap year check is done in the month validation
                if (i==2) {
                    days[i] = 29;
                }
            } 
            return days;
        },
        
        /**
		 * Converts the parameters passed to a JSON String format. 
		 * @function convertToJSONFormat
		 * @param {String} 
		 * @param {String}
		 * @return {String}
		 */
		convertToJSONFormat: function(statusCode, statusMessage) { 
			//@TODO: CR: variables names same as parameters.
			//@TODO: CR: Is statusCode a number / string? Wrong convention               
        	var statusMessage = {"statusCode" :  statusCode ,
        						 "statusMessage" : statusMessage };        	
        	return JSON.stringify(statusMessage);
		},
		
		//@TODO: CR: MEDIUM:Remove dead code
		/**
		 * @function log
		 * @param {String} Message to be logged.
		 */
        log: function(msg) {
			// un comment the below line to see the log in console window only if firebug lite is integrated along with this package       
			// console.log(msg);
			
			//By default it will call the log method if its implemented by the player caller.
			/*if(log) {
				log(msg);
			}*/
        },
        
        /**
         * 
         * @function replaceString
         * @param {String}
         * @param {String}
         * @return {String}
         */
        replaceString: function( stringToBeReplaced, stringReplacedWith ) {        
        	return  stringToBeReplaced.replace("%1", stringReplacedWith);
        },
		
        /**
         * Serializes an html fragment into a DOM node.
         * @function createDivFromHTML
         * @param {string} html An html string to be serialized into a Dom node.
         * @returns {DOMNode} Returns a Dom node from the given HTML.
         */
		createDivFromHTML: function(htmlTemplate) {
            var div = document.createElement('div');
            div.innerHTML = htmlTemplate;
            return div.firstChild;			
		}
};